#!/usr/bin/env python 
import sys 
import CORBA, PortableServer, CosNaming

class CorbaBase:
    def __init__(self,argv,orbConfig="myconfig.cfg"):
        argv.append('-ORBInitRef')
        argv.append('NameService=corbaname::localhost')
        argv.append("-ORBconfigFile")
        argv.append(orbConfig)
        
        #--------------------------------------------
        #-- Initialize the ORB and find the root POA.
        #--------------------------------------------
        self.orb = CORBA.ORB_init(argv, CORBA.ORB_ID) 
        self.poa = self.orb.resolve_initial_references("RootPOA")
        
        try:
            self.hasNameservice = True
            #-------------------------------------------------
            #-- Obtain a reference to the root naming context.
            #-------------------------------------------------
            obj = self.orb.resolve_initial_references("NameService") 
            self.rootContext = obj._narrow(CosNaming.NamingContext)
            if self.rootContext is None:
                self.hasNameservice = False
        except:
            self.hasNameservice = False

    def activatePOA(self):
        #--------------------
        #-- Activate the POA.
        #--------------------
        self.poaManager = self.poa._get_the_POAManager() 
        self.poaManager.activate()     

    def activateServant(self,servant):
        servantRef = servant._this()
        return servantRef
        
    def bind(self,name,objref):
        if self.hasNameservice:
            nameObj = [CosNaming.NameComponent(name, "")] 
            try:  
                self.rootContext.bind(nameObj, objref)              
            except CosNaming.NamingContext.AlreadyBound:  
                self.rootContext.rebind(nameObj, objref)
        else:
            raise Exception("Couldn't bind %s" % (name))
        
    def unbind(self,name):
        if self.hasNameservice:
            nameObj = [CosNaming.NameComponent(name, "")] 
            try:
                self.rootContext.unbind(nameObj)
            except CosNaming.NamingContext.NotFound, ex:
                raise ex
        
    def resolve(self,name,type):
        retObj = None
        if self.hasNameservice:
            nameObj = [CosNaming.NameComponent(name, "")] 
            try:
                resolveObj = self.rootContext.resolve(nameObj)
                #----------------------------------------------
                #-- Try to narrow the object to the given type.
                #----------------------------------------------
                retObj = resolveObj._narrow(type) 

                if retObj is None:  
                    print "Couldn't narrow object!"
                
            except CosNaming.NamingContext.NotFound, ex:
                print "Resolve failed!"                
        return retObj
        
    def getIOR(self,corbaObj):
        ior = self.orb.object_to_string(corbaObj)
        return ior
        
    def start(self):
        self.orb.run()
        
    def stop(self):
        #------------------------------------------------------------
        #-- Destroy the POA, waiting until the destruction terminates
        #------------------------------------------------------------
        self.poa.destroy(1,1)
        self.orb.destroy()
    
    
    